/*
    Crystal Space Entity Layer
    Copyright (C) 2001 by Jorrit Tyberghein
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA.
*/

#ifndef __CEL_PL_PERSIST__
#define __CEL_PL_PERSIST__

#include "cstypes.h"
#include "csutil/scf.h"
#include "csutil/util.h"

struct iCelPropertyClass;
struct iCelEntity;
struct iFile;
struct celData;

SCF_VERSION (iCelDataBuffer, 0, 0, 1);

enum
{
    CEL_PERSIST_MODE_READ = 1,
    CEL_PERSIST_MODE_WRITE = 2
};

/**
 * This interface describes persistable data.
 */
struct iCelDataBuffer : public iBase
{
  /**
   * Get a serial number for this data. This can be used
   * to check validity of the data (i.e. to compare version
   * numbers).
   */
  virtual long GetSerialNumber () const = 0;

  /**
   * Set the number of data entries.
   */
  virtual void SetDataCount (size_t cnt) = 0;

  /**
   * Get the number of data entries.
   */
  virtual size_t GetDataCount () const = 0;

  /**
   * Get a specific data entry.
   */
  virtual celData* GetData (size_t idx) const = 0;
};

SCF_VERSION (iCelPersistence, 0, 0, 1);
struct iCelPersistenceContext;

/**
 * This interface describes a module that can
 * persist CEL data.
 */
struct iCelPersistence : public iBase
{
  /**
   * Load an entity from the persistent data including
   * all property classes. 'name' is a name relevant for
   * the given type of persistence. It can be a filename
   * for example (VFS).
   */
  virtual csPtr<iCelEntity> LoadEntity (const char* name) = 0;

  /**
   * Save an entity to persistent data.
   */
  virtual bool SaveEntity (iCelEntity* entity, const char* name) = 0;

  /**
   * Create an persistence context
   */
  virtual iCelPersistenceContext* CreateContext(iBase* data, int mode,
	  bool performmapping = true) = 0;
};

SCF_VERSION (iCelPersistenceContext, 0, 0, 1);

struct iCelPersistenceContext : public iBase
{
  /**
   * Load an entity from the persitent data including all property classes.
   */
  virtual csPtr<iCelEntity> LoadEntity () = 0;

  /**
   * Save an entity to persistent data.
   */
  virtual bool SaveEntity (iCelEntity* entity) = 0;

  /* returns the ID mapped to the ID on the other side of the connection */
  virtual uint GetMappedID(iCelEntity* entity) = 0;

  /* returns the entitiy after mapping the ID */
  virtual iCelEntity* GetMappedEntity(uint id) = 0;

  /**
   * Clears internal data and buffers (like list of already saved entities)
   */
  virtual void Clear () = 0;
};

// Deprecated: Backward compatibility for long-term spelling mistake.
SCF_VERSION (iCelPersistance, 0, 0, 1);
struct iCelPersistance : public iCelPersistence {};
SCF_VERSION (iCelPersistanceContext, 0, 0, 1);
struct iCelPersistanceContext : public iCelPersistenceContext {};

#endif // __CEL_PL_PERSIST__
